import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import common as c

matplotlib.rcParams['text.usetex'] = True
matplotlib.rcParams["pdf.fonttype"] = 42
matplotlib.rcParams['font.size'] = 12

col_imag = c.col_imag
col_real = c.col_real

liwi = c.liwi
w = c.figwidth
y=1.1

def makearrowheads(fig, ax):
    xmin, xmax = ax.get_xlim()
    ymin, ymax = ax.get_ylim()
    for side in ['bottom','right','top','left']:
        ax.spines[side].set_visible(False)
    plt.xticks([]) 
    plt.yticks([])
    ax.xaxis.set_ticks_position('none') 
    ax.yaxis.set_ticks_position('none')
    dps = fig.dpi_scale_trans.inverted()
    bbox = ax.get_window_extent().transformed(dps)
    width, height = bbox.width, bbox.height
    hw = 1./40.*(ymax-ymin) 
    hl = 1./40.*(xmax-xmin)
    lw = liwi 
    ohg = 0. 
     
    yhw = hw/(ymax-ymin)*(xmax-xmin)* height/width 
    yhl = hl/(xmax-xmin)*(ymax-ymin)* width/height
     
    ax.arrow(xmin, 0, xmax-xmin, 0., fc='k', ec='k', lw = lw, 
             head_width=hw, head_length=hl, overhang = ohg, 
             length_includes_head= True, clip_on = False,
             zorder=3) 
     
    ax.arrow(0, 0, 0., ymax, fc='k', ec='k', lw = lw, 
             head_width=yhw, head_length=yhl, overhang = ohg, 
             length_includes_head= True, clip_on = False,
             zorder=3) 
 
x = np.linspace(0,1, 1000)
E1 = 0.5 * np.ones_like(x)
E2 = 0 + x

options = {
    'lw':liwi,
    }
realoptions = dict(
        lw = liwi,
        color = col_real,
        zorder=2,
        )
imagoptions = dict(
        lw = liwi,
        color = col_imag,
        ls = '--',
        zorder =1,
        )
dotoptions = dict(
        lw = liwi,
        ls = ':',
        color = 'xkcd:gray',
        zorder=1,
        )

ybot = -0.12
fs = c.fontsize
def labels(ax):
    ax.text(0, 1.05, r'$\tilde \omega$', horizontalalignment='center', fontsize=fs)
    ax.text(1.05, 0, r'$\lambda$', verticalalignment='center', fontsize=fs)

figopts={
        'figsize':(w, 0.75 * w)
        }
saveopts = {
        'bbox_inches':'tight'
        }

g = 0.1
f = lambda x: np.sin(x*0.5*np.pi)**2
Eac1 = (E1 + E2)/2. + np.sqrt( 0.25*(E1-E2)**2 + g**2  )
Eac2 = (E1 + E2)/2. - np.sqrt( 0.25*(E1-E2)**2 + g**2  )
inds_bot = Eac2>0
fig, ax = plt.subplots(**figopts)
ax.set_title('Level repulsion', y=y)
ax.plot(x, E1, **dotoptions)
ax.plot(x, E2, **dotoptions)
ax.plot(x, Eac1, **realoptions)
ax.plot(x[inds_bot], Eac2[inds_bot], **realoptions)
ax.set_ylim(ybot,1)
ax.set_xlim(0, 1)
labels(ax)
makearrowheads(fig, ax)
fig.savefig('fig1a.pdf', **saveopts)
plt.close(fig)

Eac1 = (E1 + E2)/2. + np.sqrt( 0.25*(E1-E2)**2 - g**2  + 0j )
Eac2 = (E1 + E2)/2. - np.sqrt( 0.25*(E1-E2)**2 - g**2  + 0j)
fig, ax = plt.subplots(**figopts)
ax.set_title('Level attraction', y=y)
ax.plot(x, E1, **dotoptions)
ax.plot(x, E2, **dotoptions)
ax.plot(x, Eac1.real, **realoptions)
ax.plot(x, Eac2.real, **realoptions)
ax.plot(x, Eac1.imag, **imagoptions)
ax.plot(x, Eac2.imag, **imagoptions)
ax.set_ylim(ybot,1)
ax.set_xlim(0, 1)
labels(ax)
makearrowheads(fig, ax)
fig.savefig('fig1b.pdf', **saveopts)
plt.close(fig)


