import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import common as c

matplotlib.rcParams['text.usetex'] = True

col_imag = c.col_imag
col_real = c.col_real

liwi = c.liwi

def makearrowheads(fig, ax):
    xmin, xmax = ax.get_xlim()
    ymin, ymax = ax.get_ylim()
    for side in ['bottom','right','top','left']:
        ax.spines[side].set_visible(False)
    plt.xticks([])  
    plt.yticks([])
    ax.xaxis.set_ticks_position('none') 
    ax.yaxis.set_ticks_position('none')
    dps = fig.dpi_scale_trans.inverted()
    bbox = ax.get_window_extent().transformed(dps)
    width, height = bbox.width, bbox.height
    hw = 1./40.*(ymax-ymin) 
    hl = 1./40.*(xmax-xmin)
    lw = liwi 
    ohg = 0. 
     
    yhw = hw/(ymax-ymin)*(xmax-xmin)* height/width 
    yhl = hl/(xmax-xmin)*(ymax-ymin)* width/height
     
    ax.arrow(xmin, 0, xmax-xmin, 0., fc='k', ec='k', lw = lw, 
             head_width=hw, head_length=hl, overhang = ohg, 
             length_includes_head= True, clip_on = False) 
     
    ax.arrow(0, 0, 0., ymax, fc='k', ec='k', lw = lw, 
             head_width=yhw, head_length=yhl, overhang = ohg, 
             length_includes_head= True, clip_on = False) 
 
x = np.linspace(0,1, 1000)
E1 = 0.5 * np.ones_like(x)
E2 = 0 + x

options = {
    'lw':liwi,
    }
realoptions = dict(
        lw = liwi,
        color = col_real,
        )
imagoptions = dict(
        lw = liwi,
        color = col_imag,
        ls = '--',
        zorder =1,
        )
dotoptions = dict(
        lw = 0.5,
        ls = ':',
        color = 'k',
        )


ybot = -0.1
fs = c.fontsize
def labels(ax):
    ax.text(0, 1.05, r'$\tilde \omega$', horizontalalignment='center', fontsize=fs)
    ax.text(1.05, 0, r'$\lambda$', verticalalignment='center', fontsize=fs)
    
w = c.figwidth
figopts={
        'figsize':(w, 0.75 * w)
        }
saveopts = {
        'bbox_inches':'tight'
        }

g = 0.1
d = 0.1 
d1 = 0.2-d; d2=0.24-d
sqrt = np.sqrt(-g**2 -(1./16.)*(d2-d1+2j*(E2-E1))**2)
sqrt=  np.where(E1>E2, sqrt, -sqrt)
Eac1 = (E1 + E2)/2. +1j*(d1+d2)/4. + sqrt
Eac2 = (E1 + E2)/2. +1j*(d1+d2)/4. - sqrt
fig, ax = plt.subplots(**figopts)
ax.plot(x, Eac1.real, **realoptions)
ax.plot(x, Eac2.real, **realoptions)
ax.plot(x, Eac1.imag, **imagoptions)
ax.plot(x, Eac2.imag, **imagoptions)
ax.set_ylim(ybot,1)
ax.set_xlim(0, 1)
labels(ax)
makearrowheads(fig, ax)
fig.savefig('fig1a.pdf', **saveopts)
plt.close(fig)

d1 = 0.22-d; d2=0.22-d
sqrt = np.sqrt(-g**2 +0.25*(E1-E2)**2 + 0j)
Eac1 = (E1 + E2)/2. +1j*(d1+d2)/4. + sqrt
Eac2 = (E1 + E2)/2. +1j*(d1+d2)/4. - sqrt
fig, ax = plt.subplots(**figopts)
ax.plot(x, Eac1.real, **realoptions)
ax.plot(x, Eac2.real, **realoptions)
ax.plot(x, Eac1.imag, **imagoptions)
ax.plot(x, Eac2.imag, **imagoptions)
ax.set_ylim(ybot,1)
ax.set_xlim(0, 1)
labels(ax)
makearrowheads(fig, ax)
fig.savefig('LA_symmetric_dissipation.pdf', **saveopts)
plt.close(fig)

d1 = 0.2-d; d2 = 0.8-d
sqrt = np.sqrt(-g**2 -(1./16.)*(d2-d1+2j*(E2-E1))**2)
sqrt=  np.where(E1>E2, sqrt, -sqrt)
Eac1 = (E1 + E2)/2. +1j*(d1+d2)/4. + sqrt
Eac2 = (E1 + E2)/2. +1j*(d1+d2)/4. - sqrt
fig, ax = plt.subplots(**figopts)
ax.plot(x, Eac1.real, **realoptions)
ax.plot(x, Eac2.real, **realoptions)
ax.plot(x, Eac1.imag, **imagoptions)
ax.plot(x, Eac2.imag, **imagoptions)
ax.set_ylim(ybot,1)
ax.set_xlim(0, 1)
labels(ax)
makearrowheads(fig, ax)
fig.savefig('fig2b.pdf', **saveopts)
plt.close(fig)

