import matplotlib as mpl
import numpy as np
import matplotlib.pyplot as plt
import argparse
import glob
import getdata as gd

mpl.rcParams['text.usetex'] = True
mpl.rcParams["pdf.fonttype"] = 42
mpl.rcParams['font.size'] = 12
fisi = (3, 2)
lims= [-1.5, 1.5, -0.9, 0.9]

def colorplot(data, name, vmin, vmax, LA,
        nxkeep, nykeep, omit=None, otherfunc=None, cmap='RdBu_r', ticks=[-10,0,10]):
    def imreshape(col):
        Z = (col.reshape(nx, ny))
        if omit:
            xvals = np.linspace(lims[0], lims[1], nx)
            inds = np.array(range(nx))
            inds = np.logical_and(inds>omit[0], inds<omit[1])
            inds = np.tile(inds, (ny,1)).transpose()
            Z = np.ma.masked_where(inds, Z)
        Z = Z[nxkeep[0]:nxkeep[1]]
        Z = Z[:,nykeep[0]:nykeep[1]]
        return Z.T

    pumpfreqs = imreshape(data[:,0])
    vnapoints = imreshape(data[:,1])
    pumpmid = (pumpfreqs.min() + pumpfreqs.max())/2.0
    pumpfreqs = 1e3*(pumpfreqs - pumpmid)
    vnamid = (vnapoints.min() + vnapoints.max())/2.0
    vnamin = 1e3* (vnapoints.min()-vnamid)
    vnamax = 1e3* (vnapoints.max()-vnamid)
    ran = (pumpfreqs.min(), pumpfreqs.max(), vnamin, vnamax)

    if LA:
        xlabel = '$(\Delta - \Omega_\mathrm{m}) \, / \, 2 \pi$ (MHz)'
    else:
        xlabel = '$(\Delta + \Omega_\mathrm{m}) \, / \, 2 \pi$ (MHz)'

    ylabel = 'Probe detuning (MHz)'

    class MidpointNormalize(mpl.colors.Normalize):
        def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
            self.midpoint = midpoint
            mpl.colors.Normalize.__init__(self, vmin, vmax, clip)

        def __call__(self, value, clip=None):
            result, is_scalar = self.process_value(value)
            resdat = result.data
            if self.midpoint == self.vmax:
                x, y = [self.vmin, self.vmax], [0, 0.5]
            else:
                x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
            resdat = np.interp(resdat, x, y)
            result = np.ma.array(resdat, mask=result.mask, copy=False)
            return result

    options = {
            'aspect':'auto',
            'origin':'lower',
            'extent':ran,
            'interpolation':'none',
            'vmin':vmin,
            'vmax':vmax,
            'norm':MidpointNormalize(midpoint=0.),
            }

    graphbn = name

    def colormap(Z, gname,  xlab='x', ylab='y', clab='c', cmap=cmap,
            lims=None, otherfunc=None, ticks=ticks):
        fig, ax = plt.subplots(figsize=fisi)
        fig.subplots_adjust(0,0,1,1)
        plotamp = ax.imshow(Z, **options)
        plotamp.set_cmap(cmap)
        cbar = plt.colorbar(plotamp, shrink=1)
        cbar.set_label(clab, labelpad=-5)
        cbar.set_ticks(ticks)
        if lims:
            ax.set_xlim((lims[0], lims[1]))
            ax.set_ylim((lims[2], lims[3]))
        ax.xaxis.set_ticks([-1, 0, 1])
        ax.yaxis.set_ticks([-0.5, 0, 0.5])
        ax.set_xlabel(xlab)
        ax.set_ylabel(ylab)
        if otherfunc is not None:
            otherfunc(fig,ax)
        fig.savefig(gname + '.pdf', bbox_inches='tight')

    Zamp = imreshape(data[:,2])
    Zamp = Zamp - (-51.8)
    colormap(Zamp, graphbn,
            xlabel, ylabel, 'Amplitude (dB)',
            lims=lims, otherfunc=otherfunc)

# Level repulsion
datadir =  'data/010127_vna_Pump frequency (GHz)'
data = np.loadtxt(sorted(glob.glob(datadir + '/*.dat'))[0])

ny = gd.get_a_b(datadir, 'vna', 'nop', datatype=int) 
nx = len(data)/ny 

arrowopts=dict(
        arrowstyle='<|-|>, head_width=0.1, head_length=0.1618', 
        lw=1,
        facecolor='k', shrinkA=0., shrinkB=0.,
        )
def lr_extra(fig,ax):
    delt = 0.15
    comp = 0.03
    ax.annotate('',
            xy=(0, comp-delt), xytext=(0,comp+delt),
            arrowprops=arrowopts, )
    ax.text(0.1, 0.05, "$2g$", 
            ha='left', va='center')
colorplot(data, 'fig4a', vmin=-10, vmax=0, LA=False,
        nxkeep=(0,nx-100), nykeep=(0,ny-30), omit=None,
        otherfunc=lr_extra, ticks=[-10,-5,0])

# Level attraction
datadir =  'data/182853_vna_Pump frequency (GHz)'
data = np.loadtxt(sorted(glob.glob(datadir + '/*.dat'))[0])

ny = gd.get_a_b(datadir, 'vna', 'nop', datatype=int) 
nx = len(data)/ny 

def la_extra(fig, ax): 
    delt = 0.46
    ax.annotate('',
            xy=(-delt, 0), xytext=(delt, 0),
            arrowprops=arrowopts)
    ax.text(0., 0.05, "$4g$", 
            ha='center', va='bottom')
    ax.text(0.02, -0.6, 'parametric\ninstability',
            va='center', ha='center',
            multialignment='center', size=10)
colorplot(data, 'fig4b', vmin=-10, vmax=5, LA=True,
        nxkeep=(15,nx), nykeep=(0,ny-60), omit=(391, 623),
        otherfunc=la_extra, cmap='RdBu_r', ticks=[-10, -5, 0, 5])

