import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import common as c

matplotlib.rcParams['text.usetex'] = True
matplotlib.rcParams["pdf.fonttype"] = 42
matplotlib.rcParams['font.size'] = 12

liwi = 1
w = 2.0

x = np.linspace(0., 1., 1000)
DeltaOms = 1.0*(x-0.5)
DeltaGams = 2*DeltaOms
om0 = 0.7
gam0 = 0.55
fixeddeltagam = 0.6
fixeddeltaom = fixeddeltagam / 2.0
gs = np.linspace(0, 2*fixeddeltagam/4.0, 1000)
g = 0.1

def makearrowheads(fig, ax):
    xmin, xmax = ax.get_xlim()
    ymin, ymax = ax.get_ylim()
    for side in ['bottom','right','top','left']:
        ax.spines[side].set_visible(False)
    plt.xticks([]) 
    plt.yticks([])
    ax.xaxis.set_ticks_position('none') 
    ax.yaxis.set_ticks_position('none')
    dps = fig.dpi_scale_trans.inverted()
    bbox = ax.get_window_extent().transformed(dps)
    width, height = bbox.width, bbox.height
    hw = 1./40.*(ymax-ymin) 
    hl = 1./40.*(xmax-xmin)
    lw = liwi 
    ohg = 0. 
     
    yhw = hw/(ymax-ymin)*(xmax-xmin)* height/width 
    yhl = hl/(xmax-xmin)*(ymax-ymin)* width/height
     
    ax.arrow(xmin, 0, xmax-xmin, 0., fc='k', ec='k', lw = lw, 
             head_width=hw, head_length=hl, overhang = ohg, 
             length_includes_head= True, clip_on = False,
             zorder=3) 
     
    ax.arrow(0, 0, 0., ymax, fc='k', ec='k', lw = lw, 
             head_width=yhw, head_length=yhl, overhang = ohg, 
             length_includes_head= True, clip_on = False,
             zorder=3) 

options = {
    'lw':liwi,
    }
replotoptions = dict(
        lw = liwi,
        color = c.col_real,
        zorder=3,
        clip_on=False,
        )
implotoptions = dict(
        lw = liwi,
        color = c.col_imag,
        zorder=2,
        ls = '--'
        )
dotoptions = dict(
        lw = 0.5,
        ls = ':',
        color = 'k',
        zorder=1,
        )

ybot = -0.12
fs = 14
def labels(ax, xlab = r'$\lambda$', ylab=r'$\tilde \omega$'):
    ax.text(0, 1.05, ylab, horizontalalignment='center', fontsize=fs)
    ax.text(1.05, 0, xlab, verticalalignment='center', fontsize=fs)

figopts={
        'figsize':(w, 0.75 * w)
        }
saveopts = {
        'bbox_inches':'tight'
        }

def eig_LR(om0, gam0, deltaom, deltagam, g):
    fixed = om0 - 1j*gam0/2.0  
    varia = np.sqrt(0.25*(deltaom - 1j*deltagam/2.0)**2 + g**2)
    return fixed+varia, fixed-varia
def eig_LA(om0, gam0, deltaom, deltagam, g):
    fixed = om0 + 1j*gam0/2.0  
    varia = np.sqrt(0.25*(deltaom + 1j*deltagam/2.0)**2 - g**2)
    return fixed+varia, fixed-varia

def figure(E1, E2, fname, xlab, ylab, xzero=False, title=None):
    fig, ax = plt.subplots(**figopts)
    ax.plot(x, E1.real, **replotoptions)
    ax.plot(x, E2.real, **replotoptions)
    ax.plot(x, E1.imag, **implotoptions)
    ax.plot(x, E2.imag, **implotoptions)
    ax.set_ylim(ybot,1)
    ax.set_xlim(0, 1)
    if title is not None:
        ax.set_title(title, y=1.1)
    labels(ax, xlab, ylab)
    makearrowheads(fig, ax)
    if xzero:
        ax.annotate('0',
                xy=(0.5, 0), xycoords='data', 
                size=12,
                xytext=(0.5, -0.1), ha='center', va='top',
                arrowprops=dict(arrowstyle='-', shrinkA=1, shrinkB=0),
                )
    fig.savefig(fname, **saveopts)
    plt.close(fig)

E1, E2 = eig_LR(om0, -gam0, deltaom=0, deltagam=fixeddeltagam, g=gs)
figure(E1, E2, 'fig5a.pdf', 
        r'$g$', r'$\tilde \omega^\mathrm{LR}$', title='Level repulsion')

E1, E2 = eig_LA(om0, gam0, deltaom=fixeddeltaom, deltagam=0., g=gs)
figure(E1, E2, 'fig5b.pdf', 
        r'$g$', r'$\tilde \omega^\mathrm{LA}$', title='Level attraction')

E1, E2 = eig_LR(om0, -gam0, deltaom=fixeddeltaom/10.0, 
        deltagam=fixeddeltagam, g=gs)
figure(E1, E2, 'fig5c.pdf', 
        r'$g$', r'$\tilde \omega^\mathrm{LR}$')

E1, E2 = eig_LA(om0, gam0, deltaom=fixeddeltaom, 
        deltagam=fixeddeltagam/10.0, g=gs)
figure(E1, E2, 'fig5d.pdf', 
        r'$g$', r'$\tilde \omega^\mathrm{LA}$')

E1, E2 = eig_LR(om0, -gam0, deltaom=0, deltagam=DeltaGams, g=g)
figure(E1, E2, 'fig5e.pdf', 
        r'$\Delta\Gamma$', r'$\tilde \omega^\mathrm{LR}$', xzero=True,)

E1, E2 = eig_LA(om0, gam0, deltaom=0, deltagam=DeltaGams, g=g)
inds = E1.imag < E2.imag
E1[inds], E2[inds] = E2[inds], E1[inds]
figure(E1, E2, 'fig5f.pdf', 
        r'$\Delta\Gamma$', r'$\tilde \omega^\mathrm{LA}$', xzero=True,)

